/*
 * file Example.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.Example
 *
 * (C) Copyright IBM Corporation 2005, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.stp.client.samples;

import java.io.File;

import javax.wvcm.ProviderFactory;
import javax.wvcm.Version;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyRequestItem.PropertyRequest;
import javax.wvcm.ProviderFactory.Callback;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.cc.CcDirectory;
import com.ibm.rational.wvcm.stp.cc.CcFile;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cc.CcVersion;

/**
 * This sample application uses the ClearCase CM API to
 * put a file under source control.
 */
public class Example {

    private static boolean tracing = true;

    /**
     * CM API Authentication callback. This callback is invoked when the
     * ClearCase CM API provider logs in to the CM Server.
     */
    private static class MyAuthCallback implements Callback {

        private final String serverUrl;
        private final String login;
        private final String password;

        /**
         * Construct a new authentication callback with the correct credentials
         * for the specified CM server.
         */
        public MyAuthCallback(String serverUrl, String login, String password) {
            this.serverUrl = serverUrl;
            this.login = login;
            this.password = password;
        }

        /* (non-Javadoc)
         * @see javax.wvcm.ProviderFactory$Callback#getAuthentication(java.lang.String, int)
         */
        public Authentication getAuthentication(String realm, int retries) {
            trace("Logging in to CM Server: " + serverUrl);
            trace("Login:    " + login);
            trace("Password: " + password);

            // Retry count > 0 means authentication failed.
            // Perhaps password was wrong or login domain was left out?
            myAssert(retries == 0);

            return new Authentication() {
                public String loginName() {
                    // On Windows servers, this should return domain and
                    // username separated by a backslash
                    return login;
                }
                public String password() {
                    return password;
                }
            };
        }
    }

    static void printUsageMsg() {
        System.out.println("This program expects the following arguments:");
        System.out.println("1) URL of a ClearCase CM Server");
        System.out.println("2) CM Server login");
        System.out.println("   If the CM Server is a Windows system,");
        System.out.println("   specify both the domain and login");
        System.out.println("   separated by a backslash: domain\\login.");
        System.out.println("3) CM Server password");
        System.out.println("4) Test file to add to source control");
        System.out.println("   This file must reside in a loaded, source-");
        System.out.println("   controlled directory in a ClearCase");
        System.out.println("   web view on the local machine.");
        System.out.println();
        System.out.println("For example:");
        System.out.println();
        System.out.println("  http://my_cm_server:9080/TeamWeb/services/Team my_domain\\my_login");
        System.out.println("          my_password  C:/webviews/a/b/c.txt");
    }

    /**
     * Get the ClearCase CM API provider for this example.
     * Establish authentication callback for logging in to the CM Server.
     * @return new ClearCase provider
     * @throws Exception if provider cannot be instantiated
     */
    static CcProvider getProvider(
            String serverUrl, String login, String password) throws Exception
    {
        Callback callback = new MyAuthCallback(serverUrl, login, password);
        StpProvider provider = (StpProvider) 
            ProviderFactory.createProvider(CcProvider.CC_ONLY_PROVIDER_CLASS, callback);
        
        provider.setServerUrl(serverUrl);

        return provider.ccProvider();
    }

    /**
     * Construct a CcFile proxy for the specified file.
     */
    static CcFile buildProxy(File f, CcProvider provider)
        throws WvcmException
    {
        StpLocation fileLoc = provider.filePathLocation(Domain.CLEAR_CASE, f);
        CcFile file = provider.ccFile(fileLoc);

        // Resolve the file to the correct resource type -
        // CcFile, CcDirectory, etc.
        return (CcFile) file.doResolve();
    }

    /**
     * Print some interesting properties of the specified file.
     */
    static void printProps(CcFile file) throws WvcmException {

        PropertyRequest wantedProps = new PropertyRequest(
                CcFile.IS_VERSION_CONTROLLED,
                CcFile.IS_CHECKED_OUT,
                CcFile.VERSION.nest(
                        Version.CREATION_DATE,
                        Version.VERSION_NAME));

        file = (CcFile) file.doReadProperties(wantedProps);

        trace("File is version controlled: " + file.getIsVersionControlled());
        trace("File is checked out: " + file.getIsCheckedOut());

        CcVersion version = file.getVersion();

        trace("File's checked in version: " + version.getVersionName());
        trace("Checked in version created on: " + version.getCreationDate());
    }

    /**
     * Put the specified file under ClearCase source control. The file must
     * reside in a ClearCase web view in a source-controlled directory,
     * but must not already be under source control itself.
     */
    static void addToSourceControl(CcFile res, CcDirectory parent)
        throws Exception
    {
        PropertyRequest wantedProps = new PropertyRequest(
                CcFile.IS_CHECKED_OUT,
                CcFile.IS_VERSION_CONTROLLED);

        trace("Verifying that file is NOT already source controlled...");
        res = (CcFile) res.doReadProperties(wantedProps);
        myAssert(res.getIsVersionControlled() == false);

        trace("Verifying that the parent folder IS source controlled...");
        parent = (CcDirectory) parent.doReadProperties(wantedProps);
        myAssert(parent.getIsVersionControlled() == true);

        if (parent.getIsCheckedOut()) {
            trace("Parent folder is already checked out");
        } else {
            // In order to add the test file to source control,
            // the parent directory must first be checked out.
            trace("Checking out parent folder...");
            parent.doCheckout(null, null);
        }

        trace("Adding file to source control...");
        res.doVersionControl(null);

        trace("Checking in parent folder...");
        parent.doCheckin(null, null);
    }

    static void myAssert(boolean expr) {
        if ( ! expr) {
            throw new AssertionError();
        }
    }

    static void trace(String string) {
        if (tracing) {
            System.out.println("##### " + string);
        }
    }

    public static void main(String[] args) {

        trace("Processing program arguments");
        if (args.length != 4) {
            printUsageMsg();
            return;
        }
        String CMServerUrl = args[0];
        String login = args[1];
        String password = args[2];
        File testFile = new File(args[3]);

        try {
            trace("Instantiating ClearCase CM API provider");
            CcProvider provider = getProvider(CMServerUrl, login, password);

            trace("Constructing proxy for test file");
            CcFile testRes = buildProxy(testFile, provider);

            trace("Constructing proxy for test file's parent directory");
            File parentDir = testFile.getParentFile();
            CcDirectory parent = (CcDirectory) buildProxy(parentDir, provider);

            trace("Adding test file to source control: " + testFile);
            addToSourceControl(testRes, parent);

            trace("Printing test file's properties");
            printProps(testRes);

            trace("Done");

        } catch (Exception e) {
            trace("ClearCase CM API example failed.  Printing stack trace.");
            e.printStackTrace();
        }
    }
}
